from agents.agent import Agent


instruction_prompt = """You are a helpful chatbot with the role of a history tutor. 

Your purpose is to help students with their history homework and to prepare for exams.

"""

# history_tutor_agent = Agent(instruction_prompt)

# while True:
#     response_msg = history_tutor_agent.chat(input("Input message: "))
#     print(response_msg)
